@@ -34,6 +34,11 @@
 #include "Engine/Contest/ContestResult.hpp"
 #include "FlightPhaseDetector.hpp"
 
+#if PY_MAJOR_VERSION >= 3
+    #define PyInt_FromLong PyLong_FromLong
+    #define PyInt_AsLong PyLong_AsLong
+#endif
+
 PyObject* Python::BrokenDateTimeToPy(const BrokenDateTime &datetime) {
   PyDateTime_IMPORT;
 
@@ -394,6 +399,12 @@ bool Python::PyTupleToIGCFixEnhanced(PyObject *py_fix, IGCFixEnhanced &fix) {
 }
 
 bool Python::PyStringToString(PyObject *py_string, tstring &string) {
+#if PY_MAJOR_VERSION >= 3
+  if (PyUnicode_Check(py_string)) {
+    string.assign(PyUnicode_AsUTF8(py_string));
+    return true;
+  }
+#else
   if (PyUnicode_Check(py_string)) {
     PyObject *py_string_utf8 = PyUnicode_AsUTF8String(py_string);
     string.assign(PyString_AsString(py_string_utf8));
@@ -403,6 +414,7 @@ bool Python::PyStringToString(PyObject *py_string, tstring &string) {
     string.assign(PyString_AsString(py_string));
     return true;
   }
+#endif
 
   return false;
 }
