@@ -24,10 +24,12 @@ Copyright_License {
 #include "Client.hpp"
 #include "Assemble.hpp"
 #include "Protocol.hpp"
+#include "Import.hpp"
 #include "OS/ByteOrder.hpp"
 #include "NMEA/Info.hpp"
 #include "Net/StaticSocketAddress.hxx"
 #include "Util/CRC.hpp"
+#include "Util/ConstBuffer.hxx"
 
 #ifdef HAVE_SKYLINES_TRACKING_HANDLER
 #include "IO/Async/IOThread.hpp"
@@ -113,6 +115,23 @@ SkyLinesTracking::Client::SendPing(uint16_t id)
   return SendPacket(MakePing(key, id));
 }
 
+bool
+SkyLinesTracking::Client::SendThermal(uint32_t time,
+                                      ::GeoPoint bottom_location,
+                                      int bottom_altitude,
+                                      ::GeoPoint top_location,
+                                      int top_altitude,
+                                      double lift)
+{
+  assert(socket.IsDefined());
+  assert(key != 0);
+
+  return SendPacket(MakeThermalSubmit(key, time,
+                                      bottom_location, bottom_altitude,
+                                      top_location, top_altitude,
+                                      lift));
+}
+
 #ifdef HAVE_SKYLINES_TRACKING_HANDLER
 
 bool
@@ -134,22 +153,6 @@ SkyLinesTracking::Client::SendUserNameRequest(uint32_t user_id)
   return SendPacket(MakeUserNameRequest(key, user_id));
 }
 
-static constexpr Angle
-ImportAngle(int32_t src)
-{
-  return Angle::Degrees(fixed(int32_t(FromBE32(src))) / 1000000);
-}
-
-/**
- * Convert a SkyLines #SkyLinesTracking::GeoPoint to a XCSoar
- * #::GeoPoint.
- */
-static constexpr ::GeoPoint
-ImportGeoPoint(SkyLinesTracking::GeoPoint src)
-{
-  return ::GeoPoint(ImportAngle(src.longitude), ImportAngle(src.latitude));
-}
-
 inline void
 SkyLinesTracking::Client::OnTrafficReceived(const TrafficResponsePacket &packet,
                                             size_t length)
@@ -176,7 +179,7 @@ inline void
 SkyLinesTracking::Client::OnUserNameReceived(const UserNameResponsePacket &packet,
                                              size_t length)
 {
-  if (length != sizeof(packet) + packet.name_length)
+  if (length < sizeof(packet) || length != sizeof(packet) + packet.name_length)
     return;
 
   /* the name follows the UserNameResponsePacket object */
@@ -189,6 +192,44 @@ SkyLinesTracking::Client::OnUserNameReceived(const UserNameResponsePacket &packe
   handler->OnUserName(FromBE32(packet.user_id), tname);
 }
 
+inline void
+SkyLinesTracking::Client::OnWaveReceived(const WaveResponsePacket &packet,
+                                         size_t length)
+{
+  if (length < sizeof(packet))
+    return;
+
+  const unsigned n = packet.wave_count;
+  ConstBuffer<Wave> waves((const Wave *)(&packet + 1), n);
+  if (length != sizeof(packet) + waves.size * sizeof(waves.front()))
+    return;
+
+  for (const auto &wave : waves)
+    handler->OnWave(FromBE32(wave.time),
+                    ImportGeoPoint(wave.a), ImportGeoPoint(wave.b));
+}
+
+inline void
+SkyLinesTracking::Client::OnThermalReceived(const ThermalResponsePacket &packet,
+                                            size_t length)
+{
+  if (length < sizeof(packet))
+    return;
+
+  const unsigned n = packet.thermal_count;
+  ConstBuffer<Thermal> thermals((const Thermal *)(&packet + 1), n);
+  if (length != sizeof(packet) + thermals.size * sizeof(thermals.front()))
+    return;
+
+  for (const auto &thermal : thermals)
+    handler->OnThermal(FromBE32(thermal.time),
+                       AGeoPoint(ImportGeoPoint(thermal.bottom_location),
+                                 fixed(FromBE16(thermal.bottom_altitude))),
+                       AGeoPoint(ImportGeoPoint(thermal.top_location),
+                                 fixed(FromBE16(thermal.top_altitude))),
+                       FromBE16(thermal.lift) / 256.);
+}
+
 inline void
 SkyLinesTracking::Client::OnDatagramReceived(void *data, size_t length)
 {
@@ -207,16 +248,23 @@ SkyLinesTracking::Client::OnDatagramReceived(void *data, size_t length)
   const TrafficResponsePacket &traffic = *(const TrafficResponsePacket *)data;
   const UserNameResponsePacket &user_name =
     *(const UserNameResponsePacket *)data;
+  const auto &wave = *(const WaveResponsePacket *)data;
+  const auto &thermal = *(const ThermalResponsePacket *)data;
 
   switch ((Type)FromBE16(header.type)) {
   case PING:
   case FIX:
   case TRAFFIC_REQUEST:
   case USER_NAME_REQUEST:
+  case WAVE_SUBMIT:
+  case WAVE_REQUEST:
+  case THERMAL_SUBMIT:
+  case THERMAL_REQUEST:
     break;
 
   case ACK:
-    handler->OnAck(FromBE16(ack.id));
+    if (length >= sizeof(ack))
+      handler->OnAck(FromBE16(ack.id));
     break;
 
   case TRAFFIC_RESPONSE:
@@ -226,6 +274,14 @@ SkyLinesTracking::Client::OnDatagramReceived(void *data, size_t length)
   case USER_NAME_RESPONSE:
     OnUserNameReceived(user_name, length);
     break;
+
+  case WAVE_RESPONSE:
+    OnWaveReceived(wave, length);
+    break;
+
+  case THERMAL_RESPONSE:
+    OnThermalReceived(thermal, length);
+    break;
   }
 }
 
